/*
 * (C) COPYRIGHT 2016 CRZ
 *
 * File Name : LED_Test.c
 * Author    : POOH
 * Version   : V1.0
 * Date      : 01/07/2016
 */

/* includes */

#include "hw_config.h"

#include "lwip/opt.h"
#include "lwip/init.h"
#include "netif/etharp.h"
#include "lwip/netif.h"
#include "lwip/lwip_timers.h"
#include "ethernetif.h"
#include "app_ethernet.h"
#include "httpd.h"

/* defines */

/* global variables */

struct netif gnetif;

/* functions */

static void Netif_Config(void);

/**
* @brief  Initializes the lwIP stack
* @param  None
* @retval None
*/
static void Netif_Config(void)
{
  struct ip_addr ipaddr;
  struct ip_addr netmask;
  struct ip_addr gw;
  
  /* IP address default setting */
  IP4_ADDR(&ipaddr, IP_ADDR0, IP_ADDR1, IP_ADDR2, IP_ADDR3);
  IP4_ADDR(&netmask, NETMASK_ADDR0, NETMASK_ADDR1 , NETMASK_ADDR2, NETMASK_ADDR3);
  IP4_ADDR(&gw, GW_ADDR0, GW_ADDR1, GW_ADDR2, GW_ADDR3); 
  
  /* add the network interface */    
  netif_add(&gnetif, &ipaddr, &netmask, &gw, NULL, &ethernetif_init, &ethernet_input);
  
  /*  Registers the default network interface */
  netif_set_default(&gnetif);
  
  if (netif_is_link_up(&gnetif))
  {
    /* When the netif is fully configured this function must be called */
    netif_set_up(&gnetif);
  }
  else
  {
    /* When the netif link is down this function must be called */
    netif_set_down(&gnetif);
  }
  
  /* Set the link callback function, this function is called on change of link status*/
  netif_set_link_callback(&gnetif, ethernetif_update_config);
}

void Ethernet_HTTP_Test(void)
{
    MANGO_DBG_DEFAULT_BLUE;

    /* Initialize the LwIP stack */
    lwip_init();
    MANGO_DBG("lwip_init done\n");

    /* Configure the Network interface */
    Netif_Config();
    MANGO_DBG("Netif_Config done\n");

    /* Http webserver Init */
    httpd_init();
    MANGO_DBG("httpd_init done\n");

    /* Notify user about the network interface config */
    User_notification(&gnetif);
    MANGO_DBG("User_notification done\n");

    ethernetif_set_link(&gnetif);
    MANGO_DBG("ethernetif_set_link done\n");

    /* Infinite loop */
    while (1)
    { 
        /* Read a received packet from the Ethernet buffers and send it 
        to the lwIP for handling */
        ethernetif_input(&gnetif);

        /* Handle timeouts */
        sys_check_timeouts();

#ifdef USE_DHCP
        /* handle periodic timers for LwIP */
        DHCP_Periodic_Handle(&gnetif);
#endif 
    } 
}

