/*
 * (C) COPYRIGHT 2016 CRZ
 *
 * File Name : SDMMC_Test.c
 * Author    : POOH
 * Version   : V1.0
 * Date      : 01/07/2016
 */

/* includes */

#include "hw_config.h"

/* defines */

int SDMMC_Format(void);

/* global variables */

FATFS SDFatFs;  /* File system object for SD card logical drive */
FIL MyFile;     /* File object */
char SDPath[4]; /* SD card logical drive path */

FIL g_Update_File;     /* File object */

/* functions */

void SDMMC_Initialization(void)
{
    MANGO_DBG_DEFAULT;

    /*##-1- Link the micro SD disk I/O driver ##################################*/
    if(FATFS_LinkDriver(&SD_Driver, SDPath) != 0)
    {
        MANGO_ERROR("Error FATFS_LinkDriver !!\n");
        return;
    }
    MANGO_DBG_BLUE("FATFS_LinkDriver OK\n");

    /*##-2- Register the file system object to the FatFs module ##############*/
    if(f_mount(&SDFatFs, (TCHAR const*)SDPath, 0) != FR_OK)
    {
        MANGO_ERROR("Error f_mount !!\n");
        return;
    }
    MANGO_DBG_BLUE("f_mount OK\n");
}

void SDMMC_Termination(void)
{
    MANGO_DBG_DEFAULT;

#if 1
    /*##-1- UnLink the micro SD disk I/O driver ##################################*/
    if(FATFS_UnLinkDriver(SDPath) != 0)
    {
        MANGO_ERROR("Error FATFS_UnLinkDriver !!\n");
        return;
    }
    MANGO_DBG_BLUE("FATFS_UnLinkDriver OK\n");
#endif
}

int SDMMC_Format(void)
{
    /* ## Create a FAT file system (format) on the logical drive ######### */
    /* WARNING: Formatting the uSD card will delete all content on the device */
    if(f_mkfs((TCHAR const*)SDPath, 0, 0) == FR_OK)
    {
        MANGO_DBG_BLUE("SDMMC Card Format OK\n");
        return 0;
    }
    else
    {
        MANGO_ERROR("Error FatFs Format Error !!\n");
        return -1;
    }
}

void SDMMC_Test(void)
{
    MANGO_DBG_DEFAULT_BLUE;

    FRESULT res;                                          /* FatFs function common result code */
    uint32_t byteswritten, bytesread;                     /* File write/read counts */
    uint8_t wtext[] = "This is STM32 working with FatFs"; /* File write buffer */
    uint8_t rtext[100];                                   /* File read buffer */
    uint8_t retVal;

    /* Initialize SDMMC */
    SDMMC_Initialization();

    retVal = BSP_SD_Init();
    if(MSD_OK != retVal)
    {
        HAL_Delay(10);

        MANGO_DBG_BLUE("BSP_SD_Init retry\n");
        retVal = BSP_SD_Init();
        if(MSD_OK != retVal)
        {
            MANGO_ERROR("Error BSP_SD_Init !!\n");
            return;
        }
    }
    MANGO_DBG_BLUE("BSP_SD_Init OK\n");

    if(SDMMC_Format() < 0)
    {
        MANGO_ERROR("Error Format !!\n");
        return;
    }
    MANGO_DBG_BLUE("Format OK\n");

    /*##-4- Create and Open a new text file object with write access #####*/
    if(f_open(&MyFile, "STM32.TXT", FA_CREATE_ALWAYS | FA_WRITE) != FR_OK)
    {
        /* 'STM32.TXT' file Open for write Error */
        MANGO_ERROR("Error STM32.TXT file Open for write !!\n");
        return;
    }
    MANGO_DBG_BLUE("f_open STM32.TXT OK\n");

    /*##-5- Write data to the text file ################################*/
    res = f_write(&MyFile, wtext, sizeof(wtext), (void *)&byteswritten);

    /*##-6- Close the open text file #################################*/
    if (f_close(&MyFile) != FR_OK )
    {
        MANGO_ERROR("Error STM32.TXT file close !!\n");
        return;
    }
    MANGO_DBG_BLUE("f_close STM32.TXT OK\n");

    if((byteswritten == 0) || (res != FR_OK))
    {
        /* 'STM32.TXT' file Write or EOF Error */
        MANGO_ERROR("Error STM32.TXT file Write or EOF Error !!\n");
        return;
    }

    MANGO_DBG_BLUE("STM32.TXT file Write OK\n");

    /*##-7- Open the text file object with read access ###############*/
    if(f_open(&MyFile, "STM32.TXT", FA_READ) != FR_OK)
    {
        /* 'STM32.TXT' file Open for read Error */
        MANGO_ERROR("Error STM32.TXT file Open for read Error !!\n");
        return;
    }
    MANGO_DBG_BLUE("f_open STM32.TXT OK\n");

    /*##-8- Read data from the text file ###########################*/
    res = f_read(&MyFile, rtext, sizeof(rtext), (UINT*)&bytesread);

    if((bytesread == 0) || (res != FR_OK))
    {
        /* 'STM32.TXT' file Read or EOF Error */
        MANGO_ERROR("Error STM32.TXT file Read or EOF Error !!\n");
        return;
    }
    MANGO_DBG_BLUE("f_read STM32.TXT OK\n");

    /*##-9- Close the open text file #############################*/
    f_close(&MyFile);

    /*##-10- Compare read data with the expected data ############*/
    if((bytesread != byteswritten))
    {                
        /* Read data is different from the expected data */
        MANGO_ERROR("Error Read data is different from the expected data !!\n");
        return;
    }

    while(1)
    {
        Wait_N_Seconds(1);
        MANGO_DBG("Success demo OK\n");
    }
}

