/*
 * (C) COPYRIGHT 2016 CRZ
 *
 * File Name : SRAM_Test.c
 * Author    : POOH
 * Version   : V1.0
 * Date      : 01/07/2016
 */

/* includes */

#include "hw_config.h"
#include "stm32f4_mango_sram.h"

/* defines */

#define BUFFER_SIZE         ((uint32_t)0x0100)
#define WRITE_READ_ADDR     ((uint32_t)0x0800)

/* Exported types ------------------------------------------------------------*/
typedef enum {FAILED = 0, PASSED = !FAILED} TestStatus;

/* Exported constants --------------------------------------------------------*/
#define SRAM_BANK_ADDR                 ((uint32_t)0x64000000)

/* #define SRAM_MEMORY_WIDTH            FSMC_NORSRAM_MEM_BUS_WIDTH_8  */
#define SRAM_MEMORY_WIDTH               FSMC_NORSRAM_MEM_BUS_WIDTH_16
/* #define SRAM_MEMORY_WIDTH            FSMC_NORSRAM_MEM_BUS_WIDTH_32 */

#define SRAM_TIMEOUT     ((uint32_t)0xFFFF) 

/* global variables */

SRAM_HandleTypeDef hsram;
FSMC_NORSRAM_TimingTypeDef SRAM_Timing;

/* Read/Write Buffers */
uint16_t aTxBuffer[BUFFER_SIZE];
uint16_t aRxBuffer[BUFFER_SIZE];

/* Status variables */
__IO uint32_t uwWriteReadStatus = 0;

/* Counter index */
uint32_t uwIndex = 0;

/* functions */

static void Fill_Buffer(uint16_t *pBuffer, uint32_t uwBufferLenght, uint16_t uwOffset);
static TestStatus Buffercmp(uint16_t *pBuffer1, uint16_t *pBuffer2, uint16_t BufferLength);

void SRAM_Initialization(void)
{
    MANGO_DBG_DEFAULT;

    if(0 != BSP_SRAM_Init())
    {
        MANGO_ERROR("BSP_SRAM_Init error !!\n");
    }
    else
    {
        MANGO_DBG_BLUE("BSP_SRAM_Init OK.\n");
    }
}

/**
  * @brief  Fills buffer with user predefined data.
  * @param  pBuffer: pointer on the buffer to fill
  * @param  uwBufferLenght: size of the buffer to fill
  * @param  uwOffset: first value to fill on the buffer
  * @retval None
  */
static void Fill_Buffer(uint16_t *pBuffer, uint32_t uwBufferLenght, uint16_t uwOffset)
{
  uint16_t tmpIndex = 0;

  /* Put in global buffer different values */
  for (tmpIndex = 0; tmpIndex < uwBufferLenght; tmpIndex++ )
  {
    pBuffer[tmpIndex] = tmpIndex + uwOffset;
  }
} 

/**
  * @brief  Compares two buffers.
  * @param  pBuffer1, pBuffer2: buffers to be compared.
  * @param  BufferLength: buffer's length
  * @retval PASSED: pBuffer identical to pBuffer1
  *         FAILED: pBuffer differs from pBuffer1
  */
static TestStatus Buffercmp(uint16_t* pBuffer1, uint16_t* pBuffer2, uint16_t BufferLength)
{
    int count = 0;
    while(BufferLength--)
    {
        if(*pBuffer1 != *pBuffer2)
        {
            MANGO_ERROR("Data Diff, pBuffer1: 0x%04X, pBuffer2: 0x%04X\n", * pBuffer1, * pBuffer2);
            return FAILED;
        }

        MANGO_DBG("Count: %d, Data is same.\n", ++count);

        pBuffer1++;
        pBuffer2++;
    }

    return PASSED;
}

void SRAM_Test(void)
{
    MANGO_DBG_DEFAULT_BLUE;

    /*##-2- SRAM memory read/write access ######################################*/  
    /* Fill the buffer to write */
    Fill_Buffer(aTxBuffer, BUFFER_SIZE, 0xC20F);   

    /* Write data to the SRAM memory */
    for (uwIndex = 0; uwIndex < BUFFER_SIZE; uwIndex++)
    {
        *(__IO uint16_t*) (SRAM_BANK_ADDR + WRITE_READ_ADDR + 2*uwIndex) = aTxBuffer[uwIndex];
    }    

    /* Read back data from the SRAM memory */
    for (uwIndex = 0; uwIndex < BUFFER_SIZE; uwIndex++)
    {
        aRxBuffer[uwIndex] = *(__IO uint16_t*) (SRAM_BANK_ADDR + WRITE_READ_ADDR + 2*uwIndex);
    } 

    /*##-3- Checking data integrity ############################################*/    
    uwWriteReadStatus = Buffercmp(aTxBuffer, aRxBuffer, BUFFER_SIZE);	

    if (uwWriteReadStatus != PASSED)
    {
        MANGO_ERROR("Test NOT ok.\n");
    }
    else
    {
        MANGO_DBG_BLUE("Test OK.\n");
    }
}

