/*
 * (C) COPYRIGHT 2016 CRZ
 *
 * File Name : UART6_Test.c
 * Author    : POOH
 * Version   : V1.0
 * Date      : 01/07/2016
 */

/* includes */

#include "hw_config.h"

/* defines */

/* global variables */

/* UART handler declaration */
UART_HandleTypeDef g_Uart_6_Com_Handle;

uint8_t g_Uart_6_Com_RxBuffer[UART_RXBUFFERSIZE];

/* functions */

static void UART_6_Com_Initialization(void);
static void PE5_Init(void);
static void PE5_High(void);
static void PE5_Low(void);
static void UART6_485_EN_Init(void);
static void UART6_485_EN_High(void);
static void UART6_485_EN_Low(void);

/*
 * Name : UART_6_Initialization
 */
static void UART_6_Com_Initialization(void)
{
    /*##-1- Configure the UART peripheral ######################################*/
    /* Put the USART peripheral in the Asynchronous mode (UART Mode) */
    /* UART1 configured as follow:
      - Word Length = 8 Bits
      - Stop Bit = One Stop bit
      - Parity = ODD parity
      - BaudRate = 9600 baud
      - Hardware flow control disabled (RTS and CTS signals) */
    g_Uart_6_Com_Handle.Instance          = USART_6_COM;
    g_Uart_6_Com_Handle.Init.BaudRate     = 115200;
    g_Uart_6_Com_Handle.Init.WordLength   = UART_WORDLENGTH_8B;
    g_Uart_6_Com_Handle.Init.StopBits     = UART_STOPBITS_1;
    g_Uart_6_Com_Handle.Init.Parity       = UART_PARITY_NONE;
    g_Uart_6_Com_Handle.Init.HwFlowCtl    = UART_HWCONTROL_NONE;
    g_Uart_6_Com_Handle.Init.Mode         = UART_MODE_TX_RX;
    g_Uart_6_Com_Handle.Init.OverSampling = UART_OVERSAMPLING_16;

    if(HAL_UART_Init(&g_Uart_6_Com_Handle) != HAL_OK)
    {
        /* Initialization Error */
        Error_Handler(); 
    }

    /*##-4- Put UART peripheral in reception process ###########################*/  
    if(HAL_UART_Receive_IT(&g_Uart_6_Com_Handle, g_Uart_6_Com_RxBuffer, UART_RXBUFFERSIZE) != HAL_OK)
    {
        Error_Handler();
    }
}

static CRZ_bool g_print_menu_control_flag = CRZ_TRUE;

static void print_menu_command(void)
{
    if(CRZ_TRUE == g_print_menu_control_flag)
    {
        printf("\n---------------------\n");
        printf("Press menu key\n");
        printf("---------------------\n");
        printf("1> RS232 Test\n");
        printf("2> RS485 Test\n");
        printf("---------------------\n");
        printf("x> exit\n");
        printf("---------------------\n");
    }

    g_print_menu_control_flag = CRZ_FALSE;
}

static void PE5_Init(void)
{
  GPIO_InitTypeDef  GPIO_InitStruct;
  
  /* Enable the GPIO Clock */
  __GPIOE_CLK_ENABLE();

  /* Configure the GPIO pin */
  GPIO_InitStruct.Pin = GPIO_PIN_5;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  GPIO_InitStruct.Speed = GPIO_SPEED_FAST;
  
  HAL_GPIO_Init(GPIOE, &GPIO_InitStruct);
  
  HAL_GPIO_WritePin(GPIOE, GPIO_PIN_5, GPIO_PIN_SET);
}

static void PE5_High(void)
{
  HAL_GPIO_WritePin(GPIOE, GPIO_PIN_5, GPIO_PIN_SET);
}

static void PE5_Low(void)
{
  HAL_GPIO_WritePin(GPIOE, GPIO_PIN_5, GPIO_PIN_RESET);
}

/* USART6_RTS, PG8 */
static void UART6_485_EN_Init(void)
{
  GPIO_InitTypeDef  GPIO_InitStruct;
  
  /* Enable the GPIO Clock */
  __GPIOE_CLK_ENABLE();

  /* Configure the GPIO pin */
  GPIO_InitStruct.Pin = GPIO_PIN_8;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  GPIO_InitStruct.Speed = GPIO_SPEED_FAST;
  
  HAL_GPIO_Init(GPIOG, &GPIO_InitStruct);
  
  HAL_GPIO_WritePin(GPIOG, GPIO_PIN_8, GPIO_PIN_SET);
}

static void UART6_485_EN_High(void)
{
  HAL_GPIO_WritePin(GPIOG, GPIO_PIN_8, GPIO_PIN_SET);
}

static void UART6_485_EN_Low(void)
{
  HAL_GPIO_WritePin(GPIOG, GPIO_PIN_8, GPIO_PIN_RESET);
}

void UART6_Test(void)
{
    char ch, tx_test[100];
    int count = 0;

    MANGO_DBG_DEFAULT_BLUE;

    UART_6_Com_Initialization();
    PE5_Init();
    UART6_485_EN_Init();

//PE5_High();
//UART6_485_EN_High();

    /* Infinite loop */
    while (1)
    {
        print_menu_command();

        if(CRZ_TRUE == UartQueue_Serial_Is_Empty())
        {
            continue;
        }

        ch = UartQueue_Serial_DeQueue();

        if(0 != ch)
        {
            g_print_menu_control_flag = CRZ_TRUE;

            MANGO_DBG_BLUE("%c is selected\n", ch);

            switch(ch)
            {
            case '1':
                PE5_High();

                /* Infinite loop */
                while (1)
                {
                    count ++;

                    while(CRZ_FALSE == Uart_6_Queue_Comm_Is_Empty())
                    {
                        ch = Uart_6_Queue_Comm_DeQueue();
                        MANGO_DBG_BLUE("Rx: %c (0x%02X)\n", ch, ch);
                    }

                    HAL_Delay(1000);

                    sprintf(tx_test, "Test chars %d\r\n", count);
                    HAL_UART_Transmit(&g_Uart_6_Com_Handle, (uint8_t *)tx_test, strlen(tx_test), 0xFFFF); 
                }
                break;

            case '2':
                PE5_Low();
                UART6_485_EN_Low();

                /* Infinite loop */
                while (1)
                {
                    if(CRZ_FALSE == Uart_6_Queue_Comm_Is_Empty())
                    {
                        count ++;
                        ch = Uart_6_Queue_Comm_DeQueue();
                        printf("%c", ch);
                    }

                    if(CRZ_FALSE == UartQueue_Serial_Is_Empty())
                    {
                        count ++;
                        ch = UartQueue_Serial_DeQueue();

                        sprintf(tx_test, "%c", ch);
                        UART6_485_EN_High();
                        HAL_UART_Transmit(&g_Uart_6_Com_Handle, (uint8_t *)tx_test, strlen(tx_test), 0xFFFF); 
                        UART6_485_EN_Low();
                    }
                }
                break;

            case 'x':
                return;
            }
        }
    }
}

