/*
 * (C) COPYRIGHT 2016 CRZ
 *
 * File Name : USB_Host.c
 * Author    : POOH
 * Version   : V1.0
 * Date      : 01/07/2016
 */

/* includes */

#include "hw_config.h"
#include "ff_gen_drv.h"
#include "usbh_diskio.h"
#include "usbh_def.h"

/* defines */

typedef enum {
  APPLICATION_IDLE = 0,  
  APPLICATION_START,    
  APPLICATION_RUNNING,
}MSC_ApplicationTypeDef;

/* global variables */

FATFS USBDISKFatFs;           /* File system object for USB disk logical drive */
FIL USB_Host_MyFile;                   /* File object */
char USBDISKPath[4];          /* USB Host logical drive path */
USBH_HandleTypeDef hUSB_Host; /* USB Host handle */

MSC_ApplicationTypeDef Appli_state = APPLICATION_IDLE;

/* functions */

static void USBH_UserProcess(USBH_HandleTypeDef *phost, uint8_t id);
static void MSC_Application(void);

void USB_Host_Initialization(void)
{
    MANGO_DBG_DEFAULT;

    /*##-1- Link the USB Host disk I/O driver ##################################*/
    if(FATFS_LinkDriver(&USBH_Driver, USBDISKPath) != 0)
    {
        MANGO_ERROR("FATFS_LinkDriver Fail !!\n");
        return;
    }
    MANGO_DBG_BLUE("FATFS_LinkDriver OK\n");

    /*##-2- Init Host Library ################################################*/
    USBH_Init(&hUSB_Host, USBH_UserProcess, 0);
    MANGO_DBG_BLUE("USBH_Init done\n");

    /*##-3- Add Supported Class ##############################################*/
    USBH_RegisterClass(&hUSB_Host, USBH_MSC_CLASS);
    MANGO_DBG_BLUE("USBH_RegisterClass done\n");

    /*##-4- Start Host Process ###############################################*/
    USBH_Start(&hUSB_Host);
    MANGO_DBG_BLUE("USBH_Start done\n");
}

/**
  * @brief  Main routine for Mass Storage Class
  * @param  None
  * @retval None
  */
static void MSC_Application(void)
{
    FRESULT res;                                          /* FatFs function common result code */
    uint32_t byteswritten, bytesread;                     /* File write/read counts */
    uint8_t wtext[] = "This is STM32 working with FatFs"; /* File write buffer */
    uint8_t rtext[100];                                   /* File read buffer */

    MANGO_DBG_DEFAULT;

    /* Register the file system object to the FatFs module */
    if(f_mount(&USBDISKFatFs, (TCHAR const*)USBDISKPath, 0) != FR_OK)
    {
        /* FatFs Initialization Error */
        Error_Handler();
    }

    MANGO_DBG("f_mount Ok\n");

#if 0
    /* ## Create a FAT file system (format) on the logical drive ######### */
    /* WARNING: Formatting the uSD card will delete all content on the device */
    if(f_mkfs((TCHAR const*)USBDISKPath, 0, 0) != FR_OK)
    {
        /* FatFs Initialization Error */
        Error_Handler();
    }

    MANGO_DBG("f_mkfs Ok\n");

    /* open update file */
    if(f_open(&USB_Host_MyFile, "STM32.TXT", FA_OPEN_EXISTING) != FR_OK)
    {
        MANGO_DBG_YELLOW("File %s does NOT exist !!\n", "STM32.TXT");
    }
    else
    {
        MANGO_DBG_BLUE("f_open %s OK\n", "STM32.TXT");

        /* Close the open file */
        f_close(&USB_Host_MyFile);

        /* open update file */
        if(f_unlink("STM32.TXT") != FR_OK)
        {
            MANGO_ERROR("Delete file %s Fail !!\n", "STM32.TXT");
            Error_Handler();
        }

        MANGO_DBG_BLUE("Delete %s OK\n", "STM32.TXT");
    }
#endif

    /* Create and Open a new text file object with write access */
    if(f_open(&USB_Host_MyFile, "STM32.TXT", FA_CREATE_ALWAYS | FA_WRITE) != FR_OK) 
    {
        /* 'STM32.TXT' file Open for write Error */
        Error_Handler();
    }

    MANGO_DBG("f_open Ok\n");

    /* Write data to the text file */
    res = f_write(&USB_Host_MyFile, wtext, sizeof(wtext), (void *)&byteswritten);

    if((byteswritten == 0) || (res != FR_OK))
    {
        /* 'STM32.TXT' file Write or EOF Error */
        Error_Handler();
    }

    MANGO_DBG("f_write Ok\n");

    /* Close the open text file */
    f_close(&USB_Host_MyFile);
        
    /* Open the text file object with read access */
    if(f_open(&USB_Host_MyFile, "STM32.TXT", FA_READ) != FR_OK)
    {
        /* 'STM32.TXT' file Open for read Error */
        Error_Handler();
    }

    MANGO_DBG("f_open Ok\n");

    /* Read data from the text file */
    res = f_read(&USB_Host_MyFile, rtext, sizeof(rtext), (void *)&bytesread);

    if((bytesread == 0) || (res != FR_OK))
    {
        /* 'STM32.TXT' file Read or EOF Error */
        Error_Handler();
    }

    MANGO_DBG("f_read Ok\n");

    /* Close the open text file */
    f_close(&USB_Host_MyFile);
            
    /* Compare read data with the expected data */
    if((bytesread != byteswritten))
    {                
        /* Read data is different from the expected data */
        Error_Handler();
    }

    MANGO_DBG("Compare read data Ok\n");
  
    /* Unlink the USB disk I/O driver */
    FATFS_UnLinkDriver(USBDISKPath);
}

/**
  * @brief  User Process
  * @param  phost: Host handle
  * @param  id: Host Library user message ID
  * @retval None
  */
static void USBH_UserProcess(USBH_HandleTypeDef *phost, uint8_t id)
{  
  switch(id)
  { 
  case HOST_USER_SELECT_CONFIGURATION:
    break;
    
  case HOST_USER_DISCONNECTION:
    Appli_state = APPLICATION_IDLE;
    BSP_LED_Off(LED1); 
    BSP_LED_Off(LED3);  
    f_mount(NULL, (TCHAR const*)"", 0);          
    break;
    
  case HOST_USER_CLASS_ACTIVE:
    Appli_state = APPLICATION_START;
    break;
    
  default:
    break; 
  }
}

void USB_Host_Test(void)
{
    MANGO_DBG_DEFAULT_BLUE;

    /* Initialize USB Host */
    USB_Host_Initialization();

    /*##-5- Run Application (Blocking mode) ##################################*/
    while(1)
    {  
        /* USB Host Background task */
        USBH_Process(&hUSB_Host);

        if(APPLICATION_START == Appli_state)
        {
            MANGO_DBG("Application state is START\n");
            break;
        }
        else
        {
          //  Wait_N_Seconds(1);
          //  MANGO_DBG("Application is not ready\n");
        }
    }

    MSC_Application();
    Appli_state = APPLICATION_IDLE;

    while(1)
    {
        Wait_N_Seconds(1);
        MANGO_DBG("Success demo OK\n");
    }
}

