/*
 * (C) COPYRIGHT 2015 CRZ
 *
 * File Name : hw_config.c
 * Author    : POOH
 * Version   : V1.0
 * Date      : 10/12/2015
 */

/* includes */

#include "hw_config.h"

/* defines */

/* UART handler declaration */
UART_HandleTypeDef g_Uart_1_Debug_Handle;

uint8_t g_Uart_RxBuffer[UART_RXBUFFERSIZE];

/* Private function prototypes -----------------------------------------------*/
#ifdef __GNUC__
/* With GCC/RAISONANCE, small printf (option LD Linker->Libraries->Small printf
   set to 'Yes') calls __io_putchar() */
#define PUTCHAR_PROTOTYPE int __io_putchar(int ch)
#else
#define PUTCHAR_PROTOTYPE int fputc(int ch, FILE *f)
#endif /* __GNUC__ */

/* global variables */

/* functions */

void Wait_N_Seconds(uint32_t wait_time)
{
    int i;

    for(i = 0; i < wait_time; i ++)
    {
        HAL_Delay(1000);
        MANGO_DBG("Wait (%d/%d) second\n", i+1, wait_time);
    }
}

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  None
  * @retval None
  */
void Error_Handler(void)
{
    int i;

    for(i = 0; i < 20; i ++)
    {
        BSP_LED_Toggle(LED1);
        HAL_Delay(100);
        BSP_LED_Toggle(LED2);
        HAL_Delay(100);
        BSP_LED_Toggle(LED3);
        HAL_Delay(100);
    }

    HAL_NVIC_SystemReset();
}

/*
 * Name : USART_Debug_Initialization
 */
void USART_Debug_Initialization(void)
{
    /*##-1- Configure the UART peripheral ######################################*/
    /* Put the USART peripheral in the Asynchronous mode (UART Mode) */
    /* UART1 configured as follow:
      - Word Length = 8 Bits
      - Stop Bit = One Stop bit
      - Parity = ODD parity
      - BaudRate = 9600 baud
      - Hardware flow control disabled (RTS and CTS signals) */
    g_Uart_1_Debug_Handle.Instance          = USART_1_DEBUG;
    g_Uart_1_Debug_Handle.Init.BaudRate     = 115200;
    g_Uart_1_Debug_Handle.Init.WordLength   = UART_WORDLENGTH_8B;
    g_Uart_1_Debug_Handle.Init.StopBits     = UART_STOPBITS_1;
    g_Uart_1_Debug_Handle.Init.Parity       = UART_PARITY_NONE;
    g_Uart_1_Debug_Handle.Init.HwFlowCtl    = UART_HWCONTROL_NONE;
    g_Uart_1_Debug_Handle.Init.Mode         = UART_MODE_TX_RX;
    g_Uart_1_Debug_Handle.Init.OverSampling = UART_OVERSAMPLING_16;

    if(HAL_UART_Init(&g_Uart_1_Debug_Handle) != HAL_OK)
    {
        /* Initialization Error */
        Error_Handler(); 
    }

    /*##-4- Put UART peripheral in reception process ###########################*/  
    if(HAL_UART_Receive_IT(&g_Uart_1_Debug_Handle, g_Uart_RxBuffer, UART_RXBUFFERSIZE) != HAL_OK)
    {
        Error_Handler();
    }
}

/**
  * @brief  Rx Transfer completed callback
  * @param  UartHandle: UART handle
  * @note   This example shows a simple way to report end of IT Rx transfer, and 
  *         you can add your own implementation.
  * @retval None
  */
void HAL_UART_RxCpltCallback(UART_HandleTypeDef *UartHandle)
{
#if 0
    /*##-4- Put UART peripheral in reception process ###########################*/  
    if(HAL_UART_Receive_IT(UartHandle, g_Uart_RxBuffer, UART_RXBUFFERSIZE) != HAL_OK)
    {
        Error_Handler();
    }
#endif
}

/*
 * Name : KEY_Initialization
 */
void KEY_Initialization(void)
{
    MANGO_DBG_DEFAULT;

    BSP_PB_Init(BUTTON_KEY_1, BUTTON_MODE_EXTI);
    BSP_PB_Init(BUTTON_KEY_2, BUTTON_MODE_EXTI);
}

/**
  * @brief  Retargets the C library printf function to the USART.
  * @param  None
  * @retval None
  */
PUTCHAR_PROTOTYPE
{
    /* Place your implementation of fputc here */
    /* e.g. write a character to the Debug USART and Loop until the end of transmission */

    if('\n' == ch)
    {
        int ch_tmp = '\r';

        /* Place your implementation of fputc here */
        /* e.g. write a character to the EVAL_COM1 and Loop until the end of transmission */
        HAL_UART_Transmit(&g_Uart_1_Debug_Handle, (uint8_t *)&ch_tmp, 1, 0xFFFF); 
    }

    /* Place your implementation of fputc here */
    /* e.g. write a character to the EVAL_COM1 and Loop until the end of transmission */
    HAL_UART_Transmit(&g_Uart_1_Debug_Handle, (uint8_t *)&ch, 1, 0xFFFF); 

    return ch;
}

